@@ -79,6 +79,19 @@ formatting = {
             'nodelevel': object_properties.NODE_LEVEL_BRAILLE,
             'nestinglevel': object_properties.NESTING_LEVEL_BRAILLE,
         },
+        'sound': {
+            'required': object_properties.STATE_REQUIRED_SOUND,
+            'readonly': object_properties.STATE_READ_ONLY_SOUND,
+            'insensitive': object_properties.STATE_INSENSITIVE_SOUND,
+            'checkbox': object_properties.CHECK_BOX_INDICATORS_SOUND,
+            'radiobutton': object_properties.RADIO_BUTTON_INDICATORS_SOUND,
+            'togglebutton': object_properties.TOGGLE_BUTTON_INDICATORS_SOUND,
+            'expansion': object_properties.EXPANSION_INDICATORS_SOUND,
+            'multiselect': object_properties.STATE_MULTISELECT_SOUND,
+            'clickable': object_properties.STATE_CLICKABLE_SOUND,
+            'haslongdesc': object_properties.STATE_HAS_LONGDESC_SOUND,
+            'visited': object_properties.STATE_VISITED_SOUND,
+        },
     },
 
     ####################################################################
@@ -102,7 +115,7 @@ formatting = {
             },
         'default': {
             'focused': '[]',
-            'unfocused': 'labelOrName + allTextSelection + roleName + availability + ' + MNEMONIC + ' + accelerator + childWidget',
+            'unfocused': 'labelOrName + roleName + availability + ' + MNEMONIC + ' + accelerator + childWidget',
             'basicWhereAmI': 'labelOrName + roleName',
             'detailedWhereAmI' : 'pageSummary'
             },
@@ -177,7 +190,7 @@ formatting = {
             },
         pyatspi.ROLE_FRAME: {
             'focused': 'labelOrName + roleName',
-            'unfocused': 'labelOrName + allTextSelection + roleName + unfocusedDialogCount + availability'
+            'unfocused': 'labelOrName + roleName + unfocusedDialogCount + availability'
             },
         pyatspi.ROLE_HEADER: {
             'unfocused': '(displayedText or name) + roleName',
@@ -206,8 +219,8 @@ formatting = {
             'basicWhereAmI': 'labelAndName + allTextSelection + roleName'
             },
         pyatspi.ROLE_LAYERED_PANE: {
-            'focused': 'labelAndName + allTextSelection + roleName + availability + noShowingChildren',
-            'unfocused': 'labelAndName + allTextSelection + roleName + availability + noShowingChildren',
+            'focused': 'labelAndName + roleName + availability + noShowingChildren',
+            'unfocused': 'labelAndName + roleName + availability + noShowingChildren',
             'basicWhereAmI': 'labelAndName + pause + roleName + pause + selectedItemCount + pause',
             'detailedWhereAmI': 'labelAndName + pause + roleName + pause + selectedItemCount + pause + selectedItems + pause'
             },
@@ -225,8 +238,8 @@ formatting = {
             },
         pyatspi.ROLE_LIST_ITEM: {
             'focused': 'expandableState',
-            'unfocused': 'label + displayedText + allTextSelection + pause + expandableState + pause + positionInList + pause + childWidget',
-            'basicWhereAmI': 'label + roleName + pause + name + pause + positionInList + pause + expandableState + (nodeLevel or nestingLevel) + pause'
+            'unfocused': '(labelOrName or (displayedText + allTextSelection)) + pause + expandableState + pause + positionInList + pause + listBoxItemWidgets',
+            'basicWhereAmI': 'label + roleName + pause + (name or displayedText) + pause + positionInList + pause + expandableState + (nodeLevel or nestingLevel) + pause'
             },
         pyatspi.ROLE_MATH: {
             'unfocused': 'math',
@@ -270,7 +283,7 @@ formatting = {
         },
         pyatspi.ROLE_MENU: {
             'focused': 'labelOrName + roleName',
-            'unfocused': 'labelOrName + allTextSelection + roleName + availability + ' + MNEMONIC + ' + accelerator + pause + positionInList',
+            'unfocused': 'labelOrName + roleName + availability + ' + MNEMONIC + ' + accelerator + pause + positionInList',
             'basicWhereAmI': '(ancestors or parentRoleName) + pause + labelOrName + roleName + pause + positionInList + ' + MNEMONIC
             },
         pyatspi.ROLE_MENU_ITEM: {
@@ -294,11 +307,11 @@ formatting = {
             },
         pyatspi.ROLE_PANEL: {
             'focused': 'labelAndName + roleName',
-            'unfocused': '((substring and currentLineText) or labelAndName) + roleName'
+            'unfocused': '((substring and currentLineText) or labelAndName) + roleName + pause + unrelatedLabels'
             },
         pyatspi.ROLE_PARAGRAPH: {
-            'focused': 'labelOrName + readOnly + textRole + currentLineText + allTextSelection',
-            'unfocused': 'labelOrName + readOnly + textRole + currentLineText + allTextSelection + ' + MNEMONIC,
+            'focused': 'labelOrName + readOnly + textRole + textIndentation + currentLineText + allTextSelection',
+            'unfocused': 'labelOrName + readOnly + textRole + textIndentation + currentLineText + allTextSelection + ' + MNEMONIC,
             'basicWhereAmI': 'label + readOnly + textRole + textContent + anyTextSelection + ' + MNEMONIC,
             'detailedWhereAmI': 'label + readOnly + textRole + textContentWithAttributes + anyTextSelection + ' + MNEMONIC
             },
@@ -309,8 +322,8 @@ formatting = {
             'detailedWhereAmI': 'label + readOnly + textRole + textContentWithAttributes + anyTextSelection + ' + MNEMONIC
             },
         pyatspi.ROLE_PROGRESS_BAR: {
-            'focused': 'percentage',
-            'unfocused': 'labelAndName + percentage'
+            'focused': 'progressBarIndex + progressBarValue',
+            'unfocused': 'progressBarIndex + labelAndName + progressBarValue'
             },
         pyatspi.ROLE_PUSH_BUTTON: {
             'focused': 'expandableState',
@@ -319,8 +332,8 @@ formatting = {
             },
         pyatspi.ROLE_RADIO_BUTTON: {
             'focused': 'radioState',
-            'unfocused': 'labelOrName + pause + radioState + roleName + availability + lineBreak + ' + MNEMONIC + ' + accelerator + pause + positionInList + pause',
-            'basicWhereAmI': 'radioButtonGroup + pause + labelOrName + roleName + pause + radioState + pause + positionInGroup + ' + MNEMONIC + ' + accelerator'
+            'unfocused': '(name or label) + pause + radioState + roleName + availability + lineBreak + ' + MNEMONIC + ' + accelerator + pause + positionInList + pause',
+            'basicWhereAmI': 'radioButtonGroup + pause + (name or label) + roleName + pause + radioState + pause + positionInGroup + ' + MNEMONIC + ' + accelerator'
             },
         pyatspi.ROLE_RADIO_MENU_ITEM: {
             # OpenOffice check menu items currently have a role of "menu item"
@@ -398,11 +411,11 @@ formatting = {
             },
         pyatspi.ROLE_TEAROFF_MENU_ITEM: {
             'focused': '[]',
-            'unfocused': 'labelOrName + allTextSelection + roleName + availability '
+            'unfocused': 'labelOrName + roleName + availability '
             },
         pyatspi.ROLE_TERMINAL: {
-            'focused': 'terminal',
-            'unfocused': 'terminal',
+            'focused': 'textContent',
+            'unfocused': 'textContent',
             'basicWhereAmI': 'label + readOnly + pause + textRole + pause + textContent + anyTextSelection + ' + MNEMONIC,
             'detailedWhereAmI': 'label + readOnly + pause + textRole + pause + textContentWithAttributes + anyTextSelection + ' + MNEMONIC
             },
@@ -543,7 +556,7 @@ formatting = {
                                      asString(labelAndName + value + roleName + required))]',
             },
         pyatspi.ROLE_LABEL: {
-            'unfocused': '[Text(obj, asString(label))]'
+            'unfocused': '[Text(obj, asString(labelAndName))]'
             },
         pyatspi.ROLE_LINK: {
             'unfocused': '[Link(obj, asString(currentLineText)\
@@ -566,11 +579,11 @@ formatting = {
                           or ([Component(obj,\
                                      asString(label + displayedText + expandableState + roleName + availability) + asString(accelerator))]\
                           + (nestingLevel and [Region(" " + asString(nestingLevel))])\
-                          + (childWidget and ([Region(" ")] + childWidget))))',
+                          + (listBoxItemWidgets and ([Region(" ")] + listBoxItemWidgets))))',
             'unfocused': '((substring and ' + BRAILLE_TEXT + ')\
                           or ([Component(obj, asString(labelOrName + expandableState))]\
                               + (nestingLevel and [Region(" " + asString(nestingLevel))])\
-                              + (childWidget and ([Region(" ")] + childWidget))))',
+                              + (listBoxItemWidgets and ([Region(" ")] + listBoxItemWidgets))))',
             },
         pyatspi.ROLE_MENU: {
             'focused':   '[Component(obj,\
@@ -606,7 +619,11 @@ formatting = {
         pyatspi.ROLE_PASSWORD_TEXT: {
             'unfocused': BRAILLE_TEXT
             },
-        #pyatspi.ROLE_PROGRESS_BAR: 'default'
+        pyatspi.ROLE_PROGRESS_BAR: {
+            'unfocused': '(progressBarValue and \
+                           [Component(obj, asString(labelAndName + progressBarValue + roleName + progressBarIndex))]) \
+                           or []'
+            },
         pyatspi.ROLE_PUSH_BUTTON: {
             'unfocused': '[Component(obj,\
                                      asString((labelAndName or description) + expandableState + roleName))]'
@@ -689,7 +706,130 @@ formatting = {
         #pyatspi.ROLE_TREE: 'default'
         #pyatspi.ROLE_TREE_TABLE: 'default'
         #pyatspi.ROLE_WINDOW: 'default'
-    }
+    },
+
+    ####################################################################
+    #                                                                  #
+    # Formatting for sound.                                            #
+    #                                                                  #
+    ####################################################################
+
+    'sound': {
+        'prefix': {
+            'focused': '[]',
+            'unfocused': '[]',
+            'basicWhereAmI': '[]',
+            'detailedWhereAmI': '[]'
+        },
+        'suffix': {
+            'focused': '[]',
+            'unfocused': 'clickable + hasLongDesc',
+            'basicWhereAmI': '[]',
+            'detailedWhereAmI': '[]'
+        },
+        'default': {
+            'focused': '[]',
+            'unfocused': 'roleName',
+            'basicWhereAmI': '[]',
+            'detailedWhereAmI': '[]'
+        },
+        pyatspi.ROLE_CANVAS: {
+            'unfocused': 'roleName + positionInSet',
+        },
+        pyatspi.ROLE_CHECK_BOX: {
+            'focused': 'checkedState',
+            'unfocused': 'roleName + checkedState + required + availability',
+        },
+        pyatspi.ROLE_CHECK_MENU_ITEM: {
+            'focused': 'checkedState',
+            'unfocused': 'roleName + checkedState + availability + positionInSet',
+        },
+        pyatspi.ROLE_COMBO_BOX: {
+            'focused': 'expandableState',
+            'unfocused': 'roleName + positionInSet',
+        },
+        pyatspi.ROLE_DIAL: {
+            'focused': 'percentage',
+            'unfocused': 'roleName + percentage + required + availability',
+        },
+        pyatspi.ROLE_ENTRY: {
+            'unfocused': 'roleName + readOnly + required + availability',
+        },
+        pyatspi.ROLE_HEADING: {
+            'focused': 'expandableState',
+            'unfocused': 'roleName + expandableState',
+        },
+        pyatspi.ROLE_ICON: {
+            'unfocused': 'roleName + positionInSet',
+        },
+        pyatspi.ROLE_LINK: {
+            'focused': 'expandableState',
+            'unfocused': 'roleName + visitedState + expandableState',
+        },
+        pyatspi.ROLE_LIST: {
+            'unfocused': 'roleName + multiselectableState',
+        },
+        pyatspi.ROLE_LIST_BOX: {
+            'unfocused': 'roleName + multiselectableState',
+        },
+        pyatspi.ROLE_LIST_ITEM: {
+            'focused': 'expandableState',
+            'unfocused': 'roleName + expandableState + positionInSet',
+        },
+        pyatspi.ROLE_MENU_ITEM: {
+            'focused': 'expandableState',
+            'unfocused': 'roleName + expandableState + availability + positionInSet',
+        },
+        pyatspi.ROLE_PAGE_TAB: {
+            'unfocused': 'roleName + positionInSet',
+        },
+        pyatspi.ROLE_PROGRESS_BAR: {
+            'focused': 'progressBarValue',
+            'unfocused': 'roleName + progressBarValue'
+        },
+        pyatspi.ROLE_PUSH_BUTTON: {
+            'focused': 'expandableState',
+            'unfocused': 'roleName + expandableState + availability',
+        },
+        pyatspi.ROLE_RADIO_BUTTON: {
+            'focused': 'radioState',
+            'unfocused': 'roleName + radioState + availability + positionInSet',
+        },
+        pyatspi.ROLE_RADIO_MENU_ITEM: {
+            'focused': 'radioState',
+            'unfocused': 'roleName + checkedState + availability + positionInSet',
+        },
+        pyatspi.ROLE_SCROLL_BAR: {
+            'focused': 'percentage',
+            'unfocused': 'roleName + percentage',
+        },
+        pyatspi.ROLE_SLIDER: {
+            'focused': 'percentage',
+            'unfocused': 'roleName + percentage + required + availability',
+        },
+        pyatspi.ROLE_SPIN_BUTTON: {
+            'focused': 'percentage',
+            'unfocused': 'roleName + availability + percentage + required',
+        },
+        pyatspi.ROLE_SPLIT_PANE: {
+            'focused': 'percentage',
+            'unfocused': 'roleName + percentage + availability',
+        },
+        pyatspi.ROLE_TABLE_CELL: {
+            'focused': 'expandableState',
+            'unfocused': 'roleName + expandableState',
+        },
+        pyatspi.ROLE_TABLE_ROW: {
+            'focused': 'expandableState',
+        },
+        pyatspi.ROLE_TEXT: {
+            'unfocused': 'roleName + readOnly + required + availability',
+        },
+        pyatspi.ROLE_TOGGLE_BUTTON: {
+            'focused': 'expandableState or toggleState',
+            'unfocused': 'roleName + (expandableState or toggleState) + availability',
+        },
+    },
 }
 
 class Formatting(dict):
@@ -761,23 +901,28 @@ class Formatting(dict):
           'focused', 'basicWhereAmI', etc.
         """
         try:
-            # First try to find the exact match.
-            #
-            format = self[args['mode']][args['role']][args['formatType']]
+            return self[args['mode']][args['role']][args['formatType']]
         except:
+            pass
+
+        if args.get('formatType') == 'detailedWhereAmI':
             try:
-                # Failing that, fallback to the 'unfocused' formatType
-                # for the mode and role, if it exists.
-                #
-                format = self[args['mode']][args['role']]['unfocused']
+                return self[args['mode']][args['role']]['basicWhereAmI']
             except:
-                try:
-                    # Failing that, fallback to the default for the
-                    # formatType
-                    #
-                    format = self[args['mode']]['default'][args['formatType']]
-                except:
-                    # Failing that, just used the default 'unfocused' format
-                    #
-                    format = self[args['mode']]['default']['unfocused']
-        return format
+                pass
+
+        try:
+            return self[args['mode']][args['role']]['unfocused']
+        except:
+            pass
+
+
+        try:
+            return self[args['mode']]['default'][args['formatType']]
+        except:
+            pass
+
+        try:
+            return self[args['mode']]['default']['unfocused']
+        except:
+            return []
